# Paths

A _path_ is a (specially formatted) string that locates an entry in a filesystem.

Filesystem entries are either directories (aka folders; these hold more entries) or files (these
generally hold data).

In class, we saw the command `pwd` which stands for "print working directory" and does precisely
that. When I ran that command, its output was `/home/2014/jerrin`. This is a path. Let's break it
down.

1. The very first `/` means "start at the root of the filesystem".
   The root is the topmost directory in the whole filesystem.
2. `home/` means: then go into the `home` subdirectory.
3. `2014/` means: then go into the `2014` subdirectory.
4. `jerrin` means: then go into the `jerrin` subdirectory.

In other words, a path expresses "directions" in the filesystem tree.

**Components** of a path are separated by forward slashes `/` in Unix-like OSes.
(This is in contrast with Windows, which is not Unix-like, and uses backslashes `\`.)

The path `/home/2014/jerrin` is an example of an **absolute path.**
A path is _absolute_ when it **begins at the root.**

In contrast, paths not beginning with a slash are called **relative paths.**
They are interpreted _relative to_ the current working directory.

For example, `comp-206/lesson-plan.md` is a relative path. We can't know where exactly this file is
unless we know what the current working directory is.

The interpretation of relative paths and absolute paths is similar. Only step 1 is different: in an
absolute path, it's always "start at the root directory", whereas in a relative path it's:

1. Start in the current working directory.
2. `comp-206/` means: enter the comp-206 subdirectory
3. `lesson-plan.md` means: reach the file `lesson-plan.md`

Compare the two example paths we've seen so far:
- `/home/2014/jerrin`: we understood the final component `jerrin` to be a directory
- `comp-206/lesson-plan.md`: we understood the final component `lesson-plan.md` to be a file

**Notice:** there is nothing in these paths that tells us whether the final component is a
directory or a file. We just had to know that ahead of time.

**However,** if the final component has a _trailing slash,_ then it cannot refer to a file, so it
must be a directory.

In other words, the path `/home/2014/jerrin` could also be written `/home/2014/jerrin/` and this
makes it clear that the final component is a directory. There is no way to make clear in a path
that the final component refers to a file.

## Some important absolute and relative paths

Imagine that my home directory has subdirectories and files like in this diagram:

```
/home/2014/jerrin/
  `- comp-206/
       `- lesson-plan.md
  `- comp-302/
       `- notes.txt
```

Suppose in these examples that I'm in the `comp-206` subdirectory.

- The tilde `~` is a shorthand for the path to the current user's home directory.
  For example, `~/comp-206/lesson-plan.md` is an **absolute** path as despite depending on who the
  current user is, it is independent of the current working directory.
- The dot `.` is a path to the current working directory.
  For example, `./lesson-plan.md` is a relative path, because it depends on the current working
  directory. This path would be equivalent to just writing `notes.txt`.
- The double-dot `..` is a path to the parent directory of the current working directory.
  For example, `../comp-302/notes.txt` would be a relative path.

## Remarks about using paths with programs

Whenever I used `touch`, `mkdir`, or `cd` in class, I was actually giving _relative paths!_ The
simplest kind of relative path is just be a single entry name, no slashes anywhere at all.

This means that _a path does not have to refer to a file or directory that exists._

For example, suppose I'm in the `comp-206` subdirectory from the example above.
Then, `mkdir ../comp-310` creates a directory `comp-310` _beside_ the `comp-206` directory.
The path supplied as an argument to `mkdir` did not refer to an existing directory.

## Exercises

To check your understanding in these exercises, I recommend setting up some files and directories
in your home according to the example diagram I gave above, so that you can run these commands to
see what they do.

Supposing that the `mkdir` command from the previous section was run, `cd ../comp-310` then brings
me into that directory.

In each of these questions, break down the path step-by-step as I showed above. In case of relative
paths, what is the current working directory at this point in the story?

1. What does `touch ../comp-302/exam-schedule.txt` do?
2. What does `touch ../comp-273/homework.md` do?
3. What do `mkdir ~/comp-360` followed by `mkdir ~/comp-360/assignments` do?
